#include "MyEventAction.hh"

#include "G4Event.hh"
#include "G4ios.hh"

#include "G4TrajectoryContainer.hh"
#include "G4Trajectory.hh"
#include "G4VVisManager.hh"

#include "G4SDManager.hh"
#include "G4Event.hh"
#include "G4EventManager.hh"
#include "G4HCofThisEvent.hh"
#include "G4VHitsCollection.hh"
#include "MySensitiveCalorimeter.hh"
#include "MySensitiveTracker.hh"
#include "MyTrackerHit.hh"
#include "MyTrackerHit.hh"

#include "MyAnalysis.hh"


MyEventAction::MyEventAction() {}


MyEventAction::~MyEventAction() {}


void MyEventAction::StartOfEventAction(const G4Event* evt) { 
  G4cout << "\n---> Begin of event: " << evt->GetEventID() << G4endl;
}


void MyEventAction::EndOfEventAction(const G4Event* evt){

  // Get the IDs of the collections of calorimeter hits and tracker hits. 
  G4SDManager * SDman = G4SDManager::GetSDMpointer();
  G4String colNam;
  G4int calorimeterCollID = SDman->GetCollectionID(colNam="calCollection");
  G4int trackerCollID     = SDman->GetCollectionID(colNam="trackerCollection");

  // Extract the hits: first for the calorimeter.
  G4HCofThisEvent* collectionEventHits = evt->GetHCofThisEvent();
  MyCalorimeterHitsCollection* collectionCaloHits = 0;
  G4int numCalorimeterHits = 0;
  G4double energyDepositedInActiveCalorimeterLayer = 0.0;
  if ( collectionEventHits ) {
    collectionCaloHits = dynamic_cast< MyCalorimeterHitsCollection* >
      ( collectionEventHits->GetHC(calorimeterCollID) );
  }
  if ( collectionCaloHits ) {
    numCalorimeterHits = collectionCaloHits->entries();
    for ( G4int i=0; i < numCalorimeterHits; i++ ) {
      energyDepositedInActiveCalorimeterLayer += (*collectionCaloHits)[i]->GetEdep(); 
    }
  }

  // Now the tracker hits.
  MyTrackerHitsCollection* collectionTrackerHits = 0;
  G4int numTrackerHits = 0;
  G4double energyDepositedInSiliconLayer = 0.0;
  if ( collectionEventHits ) {
    collectionTrackerHits = dynamic_cast< MyTrackerHitsCollection* >
      ( collectionEventHits->GetHC(trackerCollID) );
  }
  if ( collectionTrackerHits ) {
    numTrackerHits = collectionTrackerHits->entries();
    for ( G4int i=0; i < numTrackerHits; i++ ) {
      energyDepositedInSiliconLayer += (*collectionTrackerHits)[i]->GetEdep(); 
    }
  }

  // Fill the histograms/ntuple.
  MyAnalysis* analysis = MyAnalysis::getInstance();
  if ( numCalorimeterHits ) {
    analysis->fillCaloHistos( energyDepositedInActiveCalorimeterLayer/MeV );
  }
  if ( numTrackerHits ) {
    analysis->fillTrackerHistos( numTrackerHits, energyDepositedInSiliconLayer/MeV );
  }
  analysis->fillNtuple( energyDepositedInActiveCalorimeterLayer/MeV,
			numTrackerHits, energyDepositedInSiliconLayer/MeV );

  // Show the online histo.
  analysis->showHistogram();
  
  // Print out the information.
  G4cout << " ---  MyEventAction::EndOfEventAction  ---    event = " 
	 << evt->GetEventID() << G4endl;
  if ( numCalorimeterHits ) {
    
    G4cout << "\t Energy deposited in the Active calorimeter layer = " 
           << energyDepositedInActiveCalorimeterLayer / MeV << " (MeV)" 
           << G4endl;
  } else {
    G4cout << "\t NO HIT in the Active calorimeter layer." << G4endl; 
  }
  if ( numTrackerHits ) {
    G4cout << "\t Number of tracker hits = " << numTrackerHits << G4endl
           << "\t Energy deposited in the Silicon layer = " 
           << energyDepositedInSiliconLayer / MeV << " (MeV)" << G4endl;
  } else {
    G4cout << "\t NO HIT in the Silicon layer." << G4endl; 
  }

  // Extract the trajectories and draw them
  if ( G4VVisManager::GetConcreteInstance() ) {
    G4TrajectoryContainer* trajectoryContainer = evt->GetTrajectoryContainer();
    G4int n_trajectories = 0;
    if ( trajectoryContainer ) n_trajectories = trajectoryContainer->entries();
    for ( G4int i=0; i<n_trajectories; i++ ) { 
      G4Trajectory* trj = (G4Trajectory*) ((*(evt->GetTrajectoryContainer()))[i]);
      // trj->DrawTrajectory(50); // Draw all tracks
      if ( trj->GetCharge() != 0. ) trj->DrawTrajectory(50); // Draw only charged tracks
      // if ( trj->GetCharge() == 0. ) trj->DrawTrajectory(50); // Draw only neutral tracks
      // if ( trj->GetParticleName() == "gamma" )  
      //   trj->DrawTrajectory(50); // Draw only gammas
      // if ( trj->GetParticleName() == "neutron"  || trj->GetParticleName() == "anti_neutron" )  
      //   trj->DrawTrajectory(50); // Draw only neutrons and antineutrons
    }
  }

}

