#include "MyDetectorConstruction.hh"

#include "G4Material.hh"
#include "G4Box.hh"
#include "G4LogicalVolume.hh"
#include "G4ThreeVector.hh"
#include "G4PVPlacement.hh"
#include "globals.hh"

#include "G4VisAttributes.hh"
#include "G4Colour.hh"


MyDetectorConstruction::MyDetectorConstruction() :  
  experimentalHall_log(0), experimentalHall_phys(0),
  logicAbsorber(0), physiAbsorber(0),
  logicActive(0), physiActive(0)
{}


MyDetectorConstruction::~MyDetectorConstruction() {}


G4VPhysicalVolume* MyDetectorConstruction::Construct() {

  //------------------- materials ------------------------

  G4double a;  // atomic mass
  G4double z;  // atomic number
  G4double density, pressure, temperature;
  G4String name, symbol;
  G4int ncomponents, natoms;

  a = 39.95*g/mole;
  density = 1.782e-03*g/cm3;
  G4Material* Ar = new G4Material(name="ArgonGas", z=18.0, a, density);

  a = 55.85*g/mole;
  density = 7.87*g/cm3;
  G4Material* Fe = new G4Material(name="Fer", z=26.0, a, density);

  a = 12.01*g/mole;
  G4Element* C  = new G4Element(name="Carbon", symbol="C", z=6.0, a);

  a = 16.00*g/mole;
  G4Element* O  = new G4Element(name="Oxygen",symbol="O" , z=8.0, a);

  density  = 27.*mg/cm3;
  pressure = 50.*atmosphere;
  temperature = 325.*kelvin;
  G4Material* CO2 = new G4Material(name="CarbonicGas", density, ncomponents=2,
                                   kStateGas, temperature, pressure);
  CO2->AddElement(C, natoms=1);
  CO2->AddElement(O, natoms=2);

  //------------------- volumes --------------------------

  // --- experimental hall (world volume)
  //     beam line along x axis

  G4double expHall_x = 2.0*m;  // half dimension along x 
  G4double expHall_y = 2.0*m;  // half dimension along y
  G4double expHall_z = 2.0*m;  // half dimension along z

  G4Box* experimentalHall_box
    = new G4Box("expHall_box",expHall_x,expHall_y,expHall_z);

  experimentalHall_log = new G4LogicalVolume(experimentalHall_box, // solid 
                                             Ar,                   // material
					     "expHall_log",        // name
					     0,                    // field manager
					     0,                    // sensitive detector
					     0);                   // user limits

  experimentalHall_phys = new G4PVPlacement(0,                     // rotation
					    G4ThreeVector(),       // translation
					    "expHall",             // name
                                            experimentalHall_log,  // logical volume
					    0,                     // mother physical volume
					    false,                 // boolean operation
					    0);                    // copy number

  // --- absorber layer

  G4double xAbsorber = 50.0*cm;  // half dimension along x 
  G4double yAbsorber = 25.0*cm;  // half dimension along y
  G4double zAbsorber = 25.0*cm;  // half dimension along z

  G4Box* solidAbsorber = new G4Box("solidAbsorber",xAbsorber, yAbsorber, zAbsorber);

  logicAbsorber = new G4LogicalVolume(solidAbsorber,       // solid 
				      Fe,                  // material
				      "logicAbsorber",     // name
				      0,                   // field manager
				      0,                   // sensitive detector
				      0);                  // user limits

  physiAbsorber = new G4PVPlacement(0,                     // rotation
				    G4ThreeVector(),       // translation
                                    "physiAbsorber",       // name
				    logicAbsorber,         // logical volume
				    experimentalHall_phys, // mother physical volume
				    false,                 // boolean operation
				    0);                    // copy number

  // --- active layer

  G4double xActive =  5.0*cm;  // half dimension along x 
  G4double yActive = 25.0*cm;  // half dimension along y 
  G4double zActive = 25.0*cm;  // half dimension along z 

  G4Box* solidActive = new G4Box("solidActive",xActive, yActive, zActive);

  logicActive = new G4LogicalVolume(solidActive,           // solid 
				    CO2,                   // material
				    "logicActive",         // name
				    0,                     // field manager
				    0,                     // sensitive detector
				    0);                    // user limits

  // It is important to set properly the translation of the
  // active layer: this must be of 55 cm along x. In fact,
  // the absorber is placed at the center (0,0,0) of the 
  // experimental hall, and it extends from -50cm to +50cm;
  // the active layer, which must be placed after (x > 50cm)
  // the absorber, has a half length along x of 5 cm.
  G4double xpos = xAbsorber + xActive;
  physiActive = new G4PVPlacement(0,                       // rotation
				  G4ThreeVector(xpos,0,0), // translation
				  "physiActive",           // name
				  logicActive,             // logical volume
				  experimentalHall_phys,   // mother physical volume
				  false,                   // boolean operation
				  0);                      // copy number

  // --- Visualization attributes

  experimentalHall_log->SetVisAttributes( G4VisAttributes::Invisible );
  // The World is not visualized.

  G4VisAttributes* visAttAbsorber = new G4VisAttributes( G4Colour(1.0,1.0,1.0) );
  visAttAbsorber->SetVisibility(true);
  logicAbsorber->SetVisAttributes(visAttAbsorber);
  // The absorber layer will appear in white colour.
  // (the order of colours is: (red, green, blue) )
  
  G4VisAttributes* visAttActive = new G4VisAttributes( G4Colour(1.0,1.0,0.0) );
  visAttActive->SetVisibility(true);
  logicActive->SetVisAttributes(visAttActive);
  // The active layer will appear in yellow colour.
  
  //------------------------------------------------------

  return experimentalHall_phys;
}


