#include "MyDetectorConstruction.hh"

#include "G4Material.hh"
#include "G4Box.hh"
#include "G4LogicalVolume.hh"
#include "G4ThreeVector.hh"
#include "G4PVPlacement.hh"
#include "globals.hh"

#include "G4VisAttributes.hh"
#include "G4Colour.hh"

#include "G4Cons.hh"
#include "G4Tubs.hh"
#include "G4RotationMatrix.hh"

#include "G4Sphere.hh"
#include "G4SubtractionSolid.hh"


MyDetectorConstruction::MyDetectorConstruction() :  
  experimentalHall_log(0), experimentalHall_phys(0),
  logicAbsorber(0), physiAbsorber(0),
  logicActive(0), physiActive(0),
  logicExtraAbsorber(0), physiExtraAbsorber(0),
  logicExtraActive(0), physiExtraActive(0)
{}


MyDetectorConstruction::~MyDetectorConstruction() {}


G4VPhysicalVolume* MyDetectorConstruction::Construct() {

  //------------------- materials ------------------------

  G4double a;  // atomic mass
  G4double z;  // atomic number
  G4double density, pressure, temperature;
  G4String name, symbol;
  G4int ncomponents, natoms;

  a = 39.95*g/mole;
  density = 1.782e-03*g/cm3;
  G4Material* Ar = new G4Material(name="ArgonGas", z=18.0, a, density);

  a = 63.546*g/mole;
  density = 8.96*g/cm3;
  G4Material* Cu = new G4Material(name="Copper", z=29.0, a, density);

  a = 12.01*g/mole;
  G4Element* C  = new G4Element(name="Carbon", symbol="C", z=6.0, a);

  a = 16.00*g/mole;
  G4Element* O  = new G4Element(name="Oxygen",symbol="O" , z=8.0, a);

  density  = 27.*mg/cm3;
  pressure = 50.*atmosphere;
  temperature = 325.*kelvin;
  G4Material* CO2 = new G4Material(name="CarbonicGas", density, ncomponents=2,
                                   kStateGas, temperature, pressure);
  CO2->AddElement(C, natoms=1);
  CO2->AddElement(O, natoms=2);

  a = 207.19*g/mole;
  density = 11.35*g/cm3;
  G4Material* Lead = new G4Material(name="Lead", z=82., a, density);

  a = 28.09*g/mole;
  density = 2.33*g/cm3;
  G4Material * Silicon = new G4Material(name="Silicon", z=14., a, density);

  //------------------- volumes --------------------------

  // --- experimental hall (world volume)
  //     beam line along x axis

  G4double expHall_x = 2.0*m;  // half dimension along x 
  G4double expHall_y = 2.0*m;  // half dimension along y
  G4double expHall_z = 2.0*m;  // half dimension along z

  G4Box* experimentalHall_box
    = new G4Box("expHall_box",expHall_x,expHall_y,expHall_z);

  experimentalHall_log = new G4LogicalVolume(experimentalHall_box, // solid 
                                             Ar,                   // material
					     "expHall_log",        // name
					     0,                    // field manager
					     0,                    // sensitive detector
					     0);                   // user limits

  experimentalHall_phys = new G4PVPlacement(0,                     // rotation
					    G4ThreeVector(),       // translation
					    "expHall",             // name
                                            experimentalHall_log,  // logical volume
					    0,                     // mother physical volume
					    false,                 // boolean operation
					    0);                    // copy number

  // --- First part of the detector

  //    --- absorber layer

  G4double xAbsorber = 50.0*cm;  // half dimension along x 
  G4double yAbsorber = 25.0*cm;  // half dimension along y
  G4double zAbsorber = 25.0*cm;  // half dimension along z

  G4Box* solidAbsorber = new G4Box("solidAbsorber",xAbsorber, yAbsorber, zAbsorber);

  logicAbsorber = new G4LogicalVolume(solidAbsorber,       // solid 
				      Cu,                  // material
				      "logicAbsorber",     // name
				      0,                   // field manager
				      0,                   // sensitive detector
				      0);                  // user limits

  physiAbsorber = new G4PVPlacement(0,                     // rotation
				    G4ThreeVector(),       // translation
                                    "physiAbsorber",       // name
				    logicAbsorber,         // logical volume
				    experimentalHall_phys, // mother physical volume
				    false,                 // boolean operation
				    0);                    // copy number

  //    --- active layer

  G4double xActive =  5.0*cm;  // half dimension along x 
  G4double yActive = 25.0*cm;  // half dimension along y 
  G4double zActive = 25.0*cm;  // half dimension along z 

  G4Box* solidActive = new G4Box("solidActive",xActive, yActive, zActive);

  logicActive = new G4LogicalVolume(solidActive,           // solid 
				    CO2,                   // material
				    "logicActive",         // name
				    0,                     // field manager
				    0,                     // sensitive detector
				    0);                    // user limits

  // It is important to set properly the translation of the
  // active layer: this must be of 55 cm along x. In fact,
  // the absorber is placed at the center (0,0,0) of the 
  // experimental hall, and it extends from -50cm to +50cm;
  // the active layer, which must be placed after (x > 50cm)
  // the absorber, has a half length along x of 5 cm.
  G4double xpos = xAbsorber + xActive;
  physiActive = new G4PVPlacement(0,                       // rotation
				  G4ThreeVector(xpos,0,0), // translation
				  "physiActive",           // name
				  logicActive,             // logical volume
				  experimentalHall_phys,   // mother physical volume
				  false,                   // boolean operation
				  0);                      // copy number

  // --- Second (extra) part of the detector

  //    --- extra absorber layer

  G4double leftRadiusCone  = 25.0*cm;
  G4double rightRadiusCone = 40.0*cm;  
  G4double halfLengthCone  = 15.0*cm;

  // Define the solid Cone, exactly as usual
  G4Cons* solidCone = new G4Cons("solidCone",          // name
				 0.0,                  // left inside radius
				 leftRadiusCone,       // left outside radius
				 0.0,                  // right inside radius
				 rightRadiusCone,      // right outside radius
				 halfLengthCone,       // half length
				 0.0,                  // starting phi
				 twopi);               // final phi
						
  // Define the solid Sphere: notice that we define a full
  // sphere, not an semisphere, although in practice what
  // we are going to subtract from the solid Cone above is,
  // effectively, a semisphere, because this is the intersection
  // between the two.       
  G4Sphere* solidSphere = new G4Sphere("solidSphere",  // name
				       0.0,            // inside radius
                                       leftRadiusCone, // outside radius
                                       0.0,            // starting phi
                                       twopi,          // delta phi
                                       0.0,            // starting theta
                                       pi);            // delta theta
						    
  // Here is the boolean operation which allows to get
  // quite easily an otherwise fairly complex solid.
  // Notice that the traslation, which is the one necessary
  // for transforming the reference frame of the first solid 
  // (solidCone) to the reference frame of the second solid
  // (solidSphere), is done along the z-axis, because this
  // is the default symmetry axis in the definition of Cone
  // and Sphere solids. The rotation from z-axis to x-axis
  // will be done afterwards, in the placement operation.
  G4SubtractionSolid* solidExtraAbsorber = 	       
    new G4SubtractionSolid("solidExtraAbsorber",                 // name
			   solidCone,   // <--- the solid to be reduced
                           solidSphere, // <--- the solid to be taken away from the previous one
                           0,                                    // rotation                    
                           G4ThreeVector(0,0,halfLengthCone));   // translation

  logicExtraAbsorber = new G4LogicalVolume(solidExtraAbsorber,   // solid 
				           Lead,                 // material
				           "logicExtraAbsorber", // name
				           0,                    // field manager
				           0,                    // sensitive detector
				           0);                   // user limits

  // From now on, everything is the same as before.
  
  // Notice that a rotation must be applied, because the
  // G4Cons is defined with the cone axis along the z-axis,
  // whereas in our case we want it along the x-axis. 
  G4RotationMatrix* rm = new G4RotationMatrix;
  rm->rotateY( -90.0*deg );
  G4double gap = 1*cm;
  xpos += xActive + gap + halfLengthCone;  
  physiExtraAbsorber = new G4PVPlacement(rm,                     // rotation
					 G4ThreeVector(xpos,0,0),// translation
                                         "physiExtraAbsorber",   // name
				         logicExtraAbsorber,     // logical volume
				         experimentalHall_phys,  // mother physical volume
				         false,                  // boolean operation
				         0);                     // copy number

  //    --- extra active layer

  G4double radiusTube = 40.0*cm;
  G4double halfLengthTube = 0.250*mm;

  G4Tubs* solidExtraActive = new G4Tubs("solidExtraAbsorber",   // name
				       0.0,                     // inside radius
                                       radiusTube,              // left outside radius
                                       halfLengthTube,          // half length
                                       0.0,                     // starting phi
                                       twopi);                  // final phi

  logicExtraActive = new G4LogicalVolume(solidExtraActive,      // solid 
					 Silicon,               // material
					 "logicExtraActive",    // name
					 0,                     // field manager
					 0,                     // sensitive detector
					 0);                    // user limits

  gap = 10*cm;
  xpos += halfLengthCone + gap + halfLengthTube; 
  physiExtraActive = new G4PVPlacement(rm,                      // rotation
				       G4ThreeVector(xpos,0,0), // translation
				       "physiExtraActive",      // name
				       logicExtraActive,        // logical volume
				       experimentalHall_phys,   // mother physical volume
				       false,                   // boolean operation
				       0);                      // copy number
  
  // --- Visualization attributes

  experimentalHall_log->SetVisAttributes( G4VisAttributes::Invisible );
  // The World is not visualized.

  G4VisAttributes* visAttAbsorber = new G4VisAttributes( G4Colour(1.0,1.0,1.0) );
  visAttAbsorber->SetVisibility(true);
  visAttAbsorber->SetForceWireframe(true);
  logicAbsorber->SetVisAttributes(visAttAbsorber);
  // The absorber layer will appear in white colour.
  // (the order of colours is: (red, green, blue) )
  
  G4VisAttributes* visAttActive = new G4VisAttributes( G4Colour(1.0,1.0,0.0) );
  visAttActive->SetVisibility(true);
  visAttActive->SetForceWireframe(true);
  logicActive->SetVisAttributes(visAttActive);
  // The active layer will appear in yellow colour.
  
  logicExtraAbsorber->SetVisAttributes(visAttAbsorber);
  // Also the extra absorber layer will appear in white colour.

  logicExtraActive->SetVisAttributes(visAttActive);
  // Also the extra active layer will appear in yellow colour.
  
  //------------------------------------------------------

  return experimentalHall_phys;
}


